### Internal structure of scala.meta ASTs

We use macro annotations to generate swaths of boilerplate that make our abstract syntax trees efficient and convenient.
Here's the code that we write in [Trees.scala](/scalameta/src/main/scala/scala/meta/Trees.scala):

```
@ast class If(cond: Term, thenp: Term, elsep: Term) extends Term
```

Here's the code generated by the `@ast` macro annotation:

```
@new _root_.org.scalameta.ast.internal.astClass() @new _root_.org.scalameta.adt.Internal.leafClass() abstract trait If extends Term with _root_.scala.Product {
  @new _root_.org.scalameta.ast.internal.astField() def cond: Term;
  @new _root_.org.scalameta.ast.internal.astField() def thenp: Term;
  @new _root_.org.scalameta.ast.internal.astField() def elsep: Term;
  override type ThisType <: If
}
@new _root_.org.scalameta.ast.internal.astCompanion() @new _root_.org.scalameta.adt.Internal.leafCompanion() object If extends scala.AnyRef {
  def <init>() = {
    super.<init>();
    ()
  };
  def internalTag: _root_.scala.Int = _root_.org.scalameta.adt.Internal.calculateTag[If];
  def apply(cond: Term, thenp: Term, elsep: Term, tokens: _root_.scala.meta.Tokens = null): If = {
    def internal(cond: Term, thenp: Term, elsep: Term, tokens: _root_.scala.meta.Tokens): If = {
      _root_.org.scalameta.ast.internal.hierarchyCheck[If];
      _root_.org.scalameta.adt.Internal.nullCheck(cond);
      _root_.org.scalameta.adt.Internal.nullCheck(thenp);
      _root_.org.scalameta.adt.Internal.nullCheck(elsep);
      _root_.org.scalameta.adt.Internal.emptyCheck(cond);
      _root_.org.scalameta.adt.Internal.emptyCheck(thenp);
      _root_.org.scalameta.adt.Internal.emptyCheck(elsep);
      val node = new Impl(null, null, _root_.scala.collection.immutable.Nil, tokens)(_root_.org.scalameta.ast.internal.initParam(cond), _root_.org.scalameta.ast.internal.initParam(thenp), _root_.org.scalameta.ast.internal.initParam(elsep));
      _root_.org.scalameta.ast.internal.storeField(node._cond, cond);
      _root_.org.scalameta.ast.internal.storeField(node._thenp, thenp);
      _root_.org.scalameta.ast.internal.storeField(node._elsep, elsep);
      node
    };
    internal(cond, thenp, elsep, tokens)
  };
  @new _root_.scala.inline() final def unapply(x: If): Option[scala.Tuple3[Term, Term, Term]] = if (x.$eq$eq(null))
    _root_.scala.None
  else
    _root_.scala.Some(scala.Tuple3(x.cond, x.thenp, x.elsep));
  import _root_.scala.language.experimental.{macros=>prettyPlease};
  import _root_.scala.language.implicitConversions;
  implicit <macro> def interfaceToApi(interface: If): Api = _root_.org.scalameta.ast.internal.Macros.interfaceToApi[If, Api];
  abstract trait Api extends If {
    def $init$() = {
      ()
    };
    def copy(cond: Term = this.cond, thenp: Term = this.thenp, elsep: Term = this.elsep, tokens: _root_.scala.meta.Tokens = _root_.scala.meta.Tokens.Prototype(this)): ThisType = If.apply(cond, thenp, elsep, tokens);
    override type ThisType = If;
    override def internalTag: _root_.scala.Int = If.internalTag;
    override def productPrefix: _root_.scala.Predef.String = _root_.org.scalameta.ast.internal.productPrefix[ThisType];
    override def productArity: _root_.scala.Int = 3;
    override def productElement(n: _root_.scala.Int): Any = n match {
      case 0 => this.cond
      case 1 => this.thenp
      case 2 => this.elsep
      case _ => throw new _root_.scala.IndexOutOfBoundsException(n.toString)
    };
    override def productIterator: _root_.scala.Iterator[_root_.scala.Any] = _root_.scala.runtime.ScalaRunTime.typedProductIterator(this)
  };
  final private[If] class Impl extends Api {
    <paramaccessor> protected val internalPrototype: If = _;
    <paramaccessor> protected val internalParent: _root_.scala.meta.Tree = _;
    <paramaccessor> protected val internalScratchpad: _root_.scala.collection.immutable.Seq[Any] = _;
    <paramaccessor> protected var internalTokens: _root_.scala.meta.Tokens = _;
    <paramaccessor> var _cond: Term = _;
    <paramaccessor> var _thenp: Term = _;
    <paramaccessor> var _elsep: Term = _;
    def <init>(internalPrototype: If, internalParent: _root_.scala.meta.Tree, internalScratchpad: _root_.scala.collection.immutable.Seq[Any], internalTokens: _root_.scala.meta.Tokens)(_cond: Term, _thenp: Term, _elsep: Term) = {
      super.<init>();
      ()
    };
    private[meta] def internalCopy(prototype: _root_.scala.meta.Tree = this, parent: _root_.scala.meta.Tree = internalParent, scratchpad: _root_.scala.collection.immutable.Seq[Any] = internalScratchpad, tokens: _root_.scala.meta.Tokens = internalTokens): ThisType = new Impl(prototype.asInstanceOf[ThisType], parent, scratchpad, tokens)(_root_.org.scalameta.ast.internal.initField(this._cond), _root_.org.scalameta.ast.internal.initField(this._thenp), _root_.org.scalameta.ast.internal.initField(this._elsep));
    def parent: _root_.scala.Option[_root_.scala.meta.Tree] = if (internalParent.$bang$eq(null))
      _root_.scala.Some(internalParent)
    else
      _root_.scala.None;
    def tokens: _root_.scala.meta.Tokens = {
      internalTokens = internalTokens match {
        case null => _root_.scala.meta.internal.ui.inferTokens(this, None)
        case _root_.scala.meta.syntactic.Tokens.Prototype((proto @ _)) => _root_.scala.meta.internal.ui.inferTokens(this, Some(proto))
        case (other @ _) => other
      };
      internalTokens
    };
    def cond: Term = {
      _root_.org.scalameta.ast.internal.loadField(this._cond);
      this._cond
    };
    def thenp: Term = {
      _root_.org.scalameta.ast.internal.loadField(this._thenp);
      this._thenp
    };
    def elsep: Term = {
      _root_.org.scalameta.ast.internal.loadField(this._elsep);
      this._elsep
    }
  };
  @new _root_.org.scalameta.ast.ast() private[meta] class Quasi extends If with _root_.scala.meta.internal.ast.Quasi with Term.Quasi {
    <paramaccessor> private[this] val tree: _root_.scala.Any = _;
    <paramaccessor> private[this] val rank: _root_.scala.Int = _;
    def <init>(tree: _root_.scala.Any, rank: _root_.scala.Int) = {
      super.<init>();
      ()
    };
    override def cond: _root_.scala.Nothing = throw new _root_.scala.`package`.UnsupportedOperationException(if (this.rank.$eq$eq(0))
      "unsupported unquoting position"
    else
      "unsupported splicing position");
    override def thenp: _root_.scala.Nothing = throw new _root_.scala.`package`.UnsupportedOperationException(if (this.rank.$eq$eq(0))
      "unsupported unquoting position"
    else
      "unsupported splicing position");
    override def elsep: _root_.scala.Nothing = throw new _root_.scala.`package`.UnsupportedOperationException(if (this.rank.$eq$eq(0))
      "unsupported unquoting position"
    else
      "unsupported splicing position");
    def pt: _root_.java.lang.Class[_$60] forSome {
      <synthetic> type _$60
    } = _root_.org.scalameta.runtime.arrayClass(_root_.scala.Predef.classOf[If], this.rank)
  }
}
```

Here's the same code after typechecking with helper def macros expanded:

```
@org.scalameta.ast.internal.astClass @org.scalameta.adt.Internal.leafClass abstract trait If extends AnyRef with scala.meta.internal.ast.Term with Product {
  @org.scalameta.ast.internal.astField def cond: scala.meta.internal.ast.Term;
  @org.scalameta.ast.internal.astField def thenp: scala.meta.internal.ast.Term;
  @org.scalameta.ast.internal.astField def elsep: scala.meta.internal.ast.Term;
  override type ThisType <: scala.meta.internal.ast.Term.If
};
@@<?> @@<?> object If extends scala.AnyRef {
  def <init>(): scala.meta.internal.ast.Term.If.type = {
    If.super.<init>();
    ()
  };
  def internalTag: Int = 85;
  def apply(cond: scala.meta.internal.ast.Term, thenp: scala.meta.internal.ast.Term, elsep: scala.meta.internal.ast.Term, tokens: meta.Tokens = null): scala.meta.internal.ast.Term.If = {
    def internal(cond: scala.meta.internal.ast.Term, thenp: scala.meta.internal.ast.Term, elsep: scala.meta.internal.ast.Term, tokens: meta.Tokens): scala.meta.internal.ast.Term.If = {
      ((): Unit);
      ({
        val result$macro$294: Boolean = cond.!=(null);
        if (result$macro$294)
          scala.Tuple2.apply[Boolean, collection.immutable.Nil.type](true, scala.collection.immutable.Nil)
        else
          scala.Tuple2.apply[Boolean, List[String]](false, scala.collection.immutable.List.apply[String]("cond is equal to null"))
      } match {
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(true, _) => ()
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(false, (failures$macro$293 @ _)) => org.scalameta.invariants.InvariantFailedException.raise("cond.!=(null)", failures$macro$293, scala.collection.immutable.Map.apply[String, scala.meta.internal.ast.Term](scala.Tuple2.apply[String, scala.meta.internal.ast.Term]("cond", cond)))
      }: Unit);
      ({
        val result$macro$296: Boolean = thenp.!=(null);
        if (result$macro$296)
          scala.Tuple2.apply[Boolean, collection.immutable.Nil.type](true, scala.collection.immutable.Nil)
        else
          scala.Tuple2.apply[Boolean, List[String]](false, scala.collection.immutable.List.apply[String]("thenp is equal to null"))
      } match {
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(true, _) => ()
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(false, (failures$macro$295 @ _)) => org.scalameta.invariants.InvariantFailedException.raise("thenp.!=(null)", failures$macro$295, scala.collection.immutable.Map.apply[String, scala.meta.internal.ast.Term](scala.Tuple2.apply[String, scala.meta.internal.ast.Term]("thenp", thenp)))
      }: Unit);
      ({
        val result$macro$298: Boolean = elsep.!=(null);
        if (result$macro$298)
          scala.Tuple2.apply[Boolean, collection.immutable.Nil.type](true, scala.collection.immutable.Nil)
        else
          scala.Tuple2.apply[Boolean, List[String]](false, scala.collection.immutable.List.apply[String]("elsep is equal to null"))
      } match {
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(true, _) => ()
        case (_1: Boolean, _2: List[String])(Boolean, List[String])(false, (failures$macro$297 @ _)) => org.scalameta.invariants.InvariantFailedException.raise("elsep.!=(null)", failures$macro$297, scala.collection.immutable.Map.apply[String, scala.meta.internal.ast.Term](scala.Tuple2.apply[String, scala.meta.internal.ast.Term]("elsep", elsep)))
      }: Unit);
      ();
      ();
      ();
      val node: scala.meta.internal.ast.Term.If.Impl = new If.this.Impl(null, null, scala.collection.immutable.Nil, tokens)((null: scala.meta.internal.ast.Term), (null: scala.meta.internal.ast.Term), (null: scala.meta.internal.ast.Term));
      (node._cond_=(cond.internalCopy(cond, node, cond.internalCopy$default$3, cond.internalCopy$default$4)): Unit);
      (node._thenp_=(thenp.internalCopy(thenp, node, thenp.internalCopy$default$3, thenp.internalCopy$default$4)): Unit);
      (node._elsep_=(elsep.internalCopy(elsep, node, elsep.internalCopy$default$3, elsep.internalCopy$default$4)): Unit);
      node
    };
    internal(cond, thenp, elsep, tokens)
  };
  <synthetic> def apply$default$4: meta.Tokens = null;
  @inline final def unapply(x: scala.meta.internal.ast.Term.If): Option[(scala.meta.internal.ast.Term, scala.meta.internal.ast.Term, scala.meta.internal.ast.Term)] = if (x.==(null))
    scala.None
  else
    scala.Some.apply[(scala.meta.internal.ast.Term, scala.meta.internal.ast.Term, scala.meta.internal.ast.Term)](scala.Tuple3.apply[scala.meta.internal.ast.Term, scala.meta.internal.ast.Term, scala.meta.internal.ast.Term](x.cond, x.thenp, x.elsep));
  import scala.language.experimental.{macros=>prettyPlease};
  import scala.language.implicitConversions;
  @scala.reflect.macros.internal.macroImpl(`macro`("macroEngine" = "v7.0 (implemented in Scala 2.11.0-M8)", "isBundle" = true, "isBlackbox" = true, "className" = "org.scalameta.ast.internal$Macros", "methodName" = "interfaceToApi", "signature" = List(List(-3), List(0, 1)))[scala.meta.internal.ast.Term.If, scala.meta.internal.ast.Term.If.Api]) implicit <macro> def interfaceToApi(interface: scala.meta.internal.ast.Term.If): scala.meta.internal.ast.Term.If.Api = new org.scalameta.ast.internal.Macros(null).interfaceToApi[scala.meta.internal.ast.Term.If, scala.meta.internal.ast.Term.If.Api];
  abstract trait Api extends AnyRef with scala.meta.internal.ast.Term.If {
    def /*Api*/$init$(): Unit = {
      ()
    };
    def copy(cond: scala.meta.internal.ast.Term = this.cond, thenp: scala.meta.internal.ast.Term = this.thenp, elsep: scala.meta.internal.ast.Term = this.elsep, tokens: meta.Tokens = scala.meta.`package`.Tokens.Prototype.apply(this)): Api.this.ThisType = Term.this.If.apply(cond, thenp, elsep, tokens);
    <synthetic> def copy$default$1: scala.meta.internal.ast.Term = this.cond;
    <synthetic> def copy$default$2: scala.meta.internal.ast.Term = this.thenp;
    <synthetic> def copy$default$3: scala.meta.internal.ast.Term = this.elsep;
    <synthetic> def copy$default$4: meta.Tokens = scala.meta.`package`.Tokens.Prototype.apply(this);
    override type ThisType = scala.meta.internal.ast.Term.If;
    override def internalTag: Int = Term.this.If.internalTag;
    override def productPrefix: String = ("Term.If": String);
    override def productArity: Int = 3;
    override def productElement(n: Int): Any = n match {
      case 0 => this.cond
      case 1 => this.thenp
      case 2 => this.elsep
      case _ => throw new scala.`package`.IndexOutOfBoundsException(n.toString())
    };
    override def productIterator: Iterator[Any] = scala.runtime.ScalaRunTime.typedProductIterator[Nothing](this)
  };
  final private[If] class Impl extends AnyRef with scala.meta.internal.ast.Term.If.Api {
    <paramaccessor> private[this] val internalPrototype: scala.meta.internal.ast.Term.If = _;
    <stable> <accessor> <paramaccessor> protected def internalPrototype: scala.meta.internal.ast.Term.If = Impl.this.internalPrototype;
    <paramaccessor> private[this] val internalParent: scala.meta.Tree = _;
    <stable> <accessor> <paramaccessor> protected def internalParent: scala.meta.Tree = Impl.this.internalParent;
    <paramaccessor> private[this] val internalScratchpad: scala.collection.immutable.Seq[Any] = _;
    <stable> <accessor> <paramaccessor> protected def internalScratchpad: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad;
    <paramaccessor> private[this] var internalTokens: meta.Tokens = _;
    <accessor> <paramaccessor> protected def internalTokens: meta.Tokens = Impl.this.internalTokens;
    <accessor> <paramaccessor> protected def internalTokens_=(x$1: meta.Tokens): Unit = Impl.this.internalTokens = x$1;
    <paramaccessor> private[this] var _cond: scala.meta.internal.ast.Term = _;
    <accessor> <paramaccessor> def _cond: scala.meta.internal.ast.Term = Impl.this._cond;
    <accessor> <paramaccessor> def _cond_=(x$1: scala.meta.internal.ast.Term): Unit = Impl.this._cond = x$1;
    <paramaccessor> private[this] var _thenp: scala.meta.internal.ast.Term = _;
    <accessor> <paramaccessor> def _thenp: scala.meta.internal.ast.Term = Impl.this._thenp;
    <accessor> <paramaccessor> def _thenp_=(x$1: scala.meta.internal.ast.Term): Unit = Impl.this._thenp = x$1;
    <paramaccessor> private[this] var _elsep: scala.meta.internal.ast.Term = _;
    <accessor> <paramaccessor> def _elsep: scala.meta.internal.ast.Term = Impl.this._elsep;
    <accessor> <paramaccessor> def _elsep_=(x$1: scala.meta.internal.ast.Term): Unit = Impl.this._elsep = x$1;
    def <init>(internalPrototype: scala.meta.internal.ast.Term.If, internalParent: scala.meta.Tree, internalScratchpad: scala.collection.immutable.Seq[Any], internalTokens: meta.Tokens)(_cond: scala.meta.internal.ast.Term, _thenp: scala.meta.internal.ast.Term, _elsep: scala.meta.internal.ast.Term): scala.meta.internal.ast.Term.If.Impl = {
      Impl.super.<init>();
      ()
    };
    private[meta] def internalCopy(prototype: scala.meta.Tree = this, parent: scala.meta.Tree = Impl.this.internalParent, scratchpad: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad, tokens: meta.Tokens = Impl.this.internalTokens): Impl.this.ThisType = new If.this.Impl(prototype.asInstanceOf[Impl.this.ThisType], parent, scratchpad, tokens)((null: scala.meta.internal.ast.Term), (null: scala.meta.internal.ast.Term), (null: scala.meta.internal.ast.Term));
    override <synthetic> def internalCopy$default$1: scala.meta.Tree = this;
    override <synthetic> def internalCopy$default$2: scala.meta.Tree = Impl.this.internalParent;
    override <synthetic> def internalCopy$default$3: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad;
    override <synthetic> def internalCopy$default$4: meta.Tokens = Impl.this.internalTokens;
    def parent: Option[scala.meta.Tree] = if (Impl.this.internalParent.!=(null))
      scala.Some.apply[scala.meta.Tree](Impl.this.internalParent)
    else
      scala.None;
    def tokens: meta.Tokens = {
      Impl.this.internalTokens_=(Impl.this.internalTokens match {
        case null => scala.meta.internal.ui.inferTokens.apply(this, scala.None)
        case (underlying: scala.meta.Tree)meta.syntactic.Tokens.Prototype((proto @ _)) => scala.meta.internal.ui.inferTokens.apply(this, scala.Some.apply[scala.meta.Tree](proto))
        case (other @ _) => other
      });
      Impl.this.internalTokens
    };
    def cond: scala.meta.internal.ast.Term = {
      (if (this._cond.==(null))
        {
          scala.Predef.require(this.internalPrototype.!=(null), "internal error when initializing Impl.cond");
          this._cond_=({
            <artifact> val qual$22: scala.meta.internal.ast.Term = this.internalPrototype.cond;
            <artifact> val x$118: scala.meta.internal.ast.Term = this.internalPrototype.cond;
            <artifact> val x$119: scala.meta.internal.ast.Term.If.Impl = this;
            <artifact> val x$120: scala.collection.immutable.Seq[Any] @scala.reflect.internal.annotations.uncheckedBounds = qual$22.internalCopy$default$3;
            <artifact> val x$121: meta.Tokens = qual$22.internalCopy$default$4;
            qual$22.internalCopy(x$118, x$119, x$120, x$121)
          })
        }
      else
        (): Unit);
      this._cond
    };
    def thenp: scala.meta.internal.ast.Term = {
      (if (this._thenp.==(null))
        {
          scala.Predef.require(this.internalPrototype.!=(null), "internal error when initializing Impl.thenp");
          this._thenp_=({
            <artifact> val qual$23: scala.meta.internal.ast.Term = this.internalPrototype.thenp;
            <artifact> val x$122: scala.meta.internal.ast.Term = this.internalPrototype.thenp;
            <artifact> val x$123: scala.meta.internal.ast.Term.If.Impl = this;
            <artifact> val x$124: scala.collection.immutable.Seq[Any] @scala.reflect.internal.annotations.uncheckedBounds = qual$23.internalCopy$default$3;
            <artifact> val x$125: meta.Tokens = qual$23.internalCopy$default$4;
            qual$23.internalCopy(x$122, x$123, x$124, x$125)
          })
        }
      else
        (): Unit);
      this._thenp
    };
    def elsep: scala.meta.internal.ast.Term = {
      (if (this._elsep.==(null))
        {
          scala.Predef.require(this.internalPrototype.!=(null), "internal error when initializing Impl.elsep");
          this._elsep_=({
            <artifact> val qual$24: scala.meta.internal.ast.Term = this.internalPrototype.elsep;
            <artifact> val x$126: scala.meta.internal.ast.Term = this.internalPrototype.elsep;
            <artifact> val x$127: scala.meta.internal.ast.Term.If.Impl = this;
            <artifact> val x$128: scala.collection.immutable.Seq[Any] @scala.reflect.internal.annotations.uncheckedBounds = qual$24.internalCopy$default$3;
            <artifact> val x$129: meta.Tokens = qual$24.internalCopy$default$4;
            qual$24.internalCopy(x$126, x$127, x$128, x$129)
          })
        }
      else
        (): Unit);
      this._elsep
    }
  };
  @org.scalameta.ast.internal.astClass @org.scalameta.adt.Internal.leafClass abstract private[meta] trait Quasi extends AnyRef with scala.meta.internal.ast.Term.If with scala.meta.internal.ast.Quasi with scala.meta.internal.ast.Term.Quasi with Product {
    @org.scalameta.ast.internal.astField def tree: Any;
    @org.scalameta.ast.internal.astField def rank: Int;
    override type ThisType <: scala.meta.internal.ast.Term.If.Quasi
  };
  @@<?> @@<?> private[meta] object Quasi extends scala.AnyRef {
    def <init>(): scala.meta.internal.ast.Term.If.Quasi.type = {
      Quasi.super.<init>();
      ()
    };
    def internalTag: Int = 86;
    def apply(tree: Any, rank: Int, tokens: meta.Tokens = null): scala.meta.internal.ast.Term.If.Quasi = {
      def internal(tree: Any, rank: Int, tokens: meta.Tokens): scala.meta.internal.ast.Term.If.Quasi = {
        ((): Unit);
        ();
        ();
        ();
        ();
        val node: scala.meta.internal.ast.Term.If.Quasi.Impl = new Quasi.this.Impl(null, null, scala.collection.immutable.Nil, tokens)((tree: Any), (rank: Int));
        ((): Unit);
        ((): Unit);
        node
      };
      internal(tree, rank, tokens)
    };
    <synthetic> def apply$default$3: meta.Tokens = null;
    @inline final def unapply(x: scala.meta.internal.ast.Term.If.Quasi): Option[(Any, Int)] = if (x.==(null))
      scala.None
    else
      scala.Some.apply[(Any, Int)](scala.Tuple2.apply[Any, Int](x.tree, x.rank));
    import scala.language.experimental.{macros=>prettyPlease};
    import scala.language.implicitConversions;
    @scala.reflect.macros.internal.macroImpl(`macro`("macroEngine" = "v7.0 (implemented in Scala 2.11.0-M8)", "isBundle" = true, "isBlackbox" = true, "className" = "org.scalameta.ast.internal$Macros", "methodName" = "interfaceToApi", "signature" = List(List(-3), List(0, 1)))[scala.meta.internal.ast.Term.If.Quasi, scala.meta.internal.ast.Term.If.Quasi.Api]) implicit <macro> def interfaceToApi(interface: scala.meta.internal.ast.Term.If.Quasi): scala.meta.internal.ast.Term.If.Quasi.Api = new org.scalameta.ast.internal.Macros(null).interfaceToApi[scala.meta.internal.ast.Term.If.Quasi, scala.meta.internal.ast.Term.If.Quasi.Api];
    abstract trait Api extends AnyRef with scala.meta.internal.ast.Term.If.Quasi {
      def /*Api*/$init$(): Unit = {
        ()
      };
      override def cond: Nothing = throw new scala.`package`.UnsupportedOperationException(if (this.rank.==(0))
        "unsupported unquoting position"
      else
        "unsupported splicing position");
      override def thenp: Nothing = throw new scala.`package`.UnsupportedOperationException(if (this.rank.==(0))
        "unsupported unquoting position"
      else
        "unsupported splicing position");
      override def elsep: Nothing = throw new scala.`package`.UnsupportedOperationException(if (this.rank.==(0))
        "unsupported unquoting position"
      else
        "unsupported splicing position");
      def pt: Class[_] = org.scalameta.runtime.`package`.arrayClass(classOf[scala.meta.internal.ast.Term$$If], this.rank);
      def copy(tree: Any = this.tree, rank: Int = this.rank, tokens: meta.Tokens = scala.meta.`package`.Tokens.Prototype.apply(this)): Api.this.ThisType = If.this.Quasi.apply(tree, rank, tokens);
      <synthetic> def copy$default$1: Any = this.tree;
      <synthetic> def copy$default$2: Int = this.rank;
      <synthetic> def copy$default$3: meta.Tokens = scala.meta.`package`.Tokens.Prototype.apply(this);
      override type ThisType = scala.meta.internal.ast.Term.If.Quasi;
      override def internalTag: Int = If.this.Quasi.internalTag;
      override def productPrefix: String = ("Term.If.Quasi": String);
      override def productArity: Int = 2;
      override def productElement(n: Int): Any = n match {
        case 0 => this.tree
        case 1 => this.rank
        case _ => throw new scala.`package`.IndexOutOfBoundsException(n.toString())
      };
      override def productIterator: Iterator[Any] = scala.runtime.ScalaRunTime.typedProductIterator[Nothing](this)
    };
    final private[Quasi] class Impl extends AnyRef with scala.meta.internal.ast.Term.If.Quasi.Api {
      <paramaccessor> private[this] val internalPrototype: scala.meta.internal.ast.Term.If.Quasi = _;
      <stable> <accessor> <paramaccessor> protected def internalPrototype: scala.meta.internal.ast.Term.If.Quasi = Impl.this.internalPrototype;
      <paramaccessor> private[this] val internalParent: scala.meta.Tree = _;
      <stable> <accessor> <paramaccessor> protected def internalParent: scala.meta.Tree = Impl.this.internalParent;
      <paramaccessor> private[this] val internalScratchpad: scala.collection.immutable.Seq[Any] = _;
      <stable> <accessor> <paramaccessor> protected def internalScratchpad: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad;
      <paramaccessor> private[this] var internalTokens: meta.Tokens = _;
      <accessor> <paramaccessor> protected def internalTokens: meta.Tokens = Impl.this.internalTokens;
      <accessor> <paramaccessor> protected def internalTokens_=(x$1: meta.Tokens): Unit = Impl.this.internalTokens = x$1;
      <paramaccessor> private[this] var _tree: Any = _;
      <accessor> <paramaccessor> def _tree: Any = Impl.this._tree;
      <accessor> <paramaccessor> def _tree_=(x$1: Any): Unit = Impl.this._tree = x$1;
      <paramaccessor> private[this] var _rank: Int = _;
      <accessor> <paramaccessor> def _rank: Int = Impl.this._rank;
      <accessor> <paramaccessor> def _rank_=(x$1: Int): Unit = Impl.this._rank = x$1;
      def <init>(internalPrototype: scala.meta.internal.ast.Term.If.Quasi, internalParent: scala.meta.Tree, internalScratchpad: scala.collection.immutable.Seq[Any], internalTokens: meta.Tokens)(_tree: Any, _rank: Int): scala.meta.internal.ast.Term.If.Quasi.Impl = {
        Impl.super.<init>();
        ()
      };
      private[meta] def internalCopy(prototype: scala.meta.Tree = this, parent: scala.meta.Tree = Impl.this.internalParent, scratchpad: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad, tokens: meta.Tokens = Impl.this.internalTokens): Impl.this.ThisType = new Quasi.this.Impl(prototype.asInstanceOf[Impl.this.ThisType], parent, scratchpad, tokens)((this._tree: Any), (this._rank: Int));
      override <synthetic> def internalCopy$default$1: scala.meta.Tree = this;
      override <synthetic> def internalCopy$default$2: scala.meta.Tree = Impl.this.internalParent;
      override <synthetic> def internalCopy$default$3: scala.collection.immutable.Seq[Any] = Impl.this.internalScratchpad;
      override <synthetic> def internalCopy$default$4: meta.Tokens = Impl.this.internalTokens;
      def parent: Option[scala.meta.Tree] = if (Impl.this.internalParent.!=(null))
        scala.Some.apply[scala.meta.Tree](Impl.this.internalParent)
      else
        scala.None;
      def tokens: meta.Tokens = {
        Impl.this.internalTokens_=(Impl.this.internalTokens match {
          case null => scala.meta.internal.ui.inferTokens.apply(this, scala.None)
          case (underlying: scala.meta.Tree)meta.syntactic.Tokens.Prototype((proto @ _)) => scala.meta.internal.ui.inferTokens.apply(this, scala.Some.apply[scala.meta.Tree](proto))
          case (other @ _) => other
        });
        Impl.this.internalTokens
      };
      def tree: Any = {
        ((): Unit);
        this._tree
      };
      def rank: Int = {
        ((): Unit);
        this._rank
      }
    }
  }
};
```
